/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VORLIB_FILELIST_H
#define VORLIB_FILELIST_H

#include <string>
#include "searchtree.h"
#include "stringlist.h"
#include "exception.h"

namespace vorlib {

class filelist_e : public exception {
public:
  filelist_e(const std::string &message = "", int id = 0);
};

/*
 ERROR ids:
 1: source file not found
 2: source file not closed
 3: section duplicated
 4: ']' missing in section header
 5: space missing after file name
 6: path missing
 7: path defined outside section
 8: path name duplicated
 9: section or file name not found
 10: base path does not have enough slashes
 11: empty path
*/

class filelist {
private:
  class section {
  public:
    balanced_search_tree<std::string, std::string> files;
    bool add(const std::string &name, const std::string &path, int convert);
    const std::string& operator[](const std::string &name) const;
    stringlist all() const;
  };
  
  balanced_search_tree<std::string, section> sections;//all data
  std::string connectpath(const std::string &path) const;//connects it with base
  bool isabsolute(const std::string &path) const;//returns wether the path is absolute or not (relative)
public:

  void load(const std::string &file, int conversion = 0);//load file - add new files to list
  void clear();//forget all files

  std::string getpath(const std::string &section, const std::string &name) const;//get path to one file
  stringlist getsection(const std::string &section) const;//get whole section
  std::string basepath;//it will be added to every wanted file as the base path, with ../ in file path working
                   /*with / (\) at the begining*/
};

/*converts:
  0: no convert
  1: / -> \
  2: \ -> /
*/
  

/*
[section_name_1]
file1 path_to_file
file2 path_to_file
     ^            ^
  one space     no spaces (if it's not part of the file path)
#comment

...

*/

}

#endif
